@@ -35,8 +35,7 @@
 			       (string=? text (substring sym 0 text-len)))
 		      (if match
 			  ;; more than one match, save the longest text that all syms match
-			  (let* ((match-len (string-length match))
-				 (min-len (min match-len sym-len)))
+			  (let ((min-len (min (string-length match) sym-len)))
 			    (do ((i text-len (+ i 1)))
 				((or (= i min-len)
 				     (not (char=? (match i) (sym i))))
@@ -421,6 +420,7 @@
 	    
 	    ;; to enable mouse click coords (in xterm anyway), (format *stderr* "~C[?9h" #\escape)
 	    ;;    disable: (format *stderr* "~C[?9l" #\escape)
+	    ;;    this is X10_MOUSE -- probably better to use SET_ANY_EVENT_MOUSE 1003
 	    ;; while enabled, mouse selection instead sends coords to repl (so it's annoying)
 	    ;;    also it's sticky!  exit repl does not clear this flag so mouse is effectively dead
 	    ;; upon click, we get ESC [ M b x y (b = button info, x/y are coords starting at 32=0 -- 6 chars received)
